#include "View.hpp"
#include "util.hpp"

#include <iostream>

namespace mcpaint::gui {

view::view(std::initializer_list<view*> children) : children(std::move(children)) {
}

view::~view() {
	for (auto& child : this->children)
		delete child;
}

void view::updateSelf(float deltaTime, const view* const parent) {
	Vector2 parentSize  = parent != nullptr ? parent->size() : Vector2{static_cast<float>(GetScreenWidth()), static_cast<float>(GetScreenHeight())};
	float parentPadding = parent != nullptr ? parent->spec().padding : 0;

	float width  = std::holds_alternative<float>(this->spec_.size.first) ? std::get<float>(this->spec_.size.first) * parentSize.x / 100 : std::get<int>(this->spec_.size.first);
	float height = std::holds_alternative<float>(this->spec_.size.second) ? std::get<float>(this->spec_.size.second) * parentSize.y / 100 : std::get<int>(this->spec_.size.second);

	width  -= this->spec_.margin * 2;
	height -= this->spec_.margin * 2;

	this->size_   = {width, height};
	this->margin_ = this->spec_.margin;
}

void view::updateChildren(float deltaTime) {
	for (auto& child : this->children)
		child->update(deltaTime, this);
}

void view::update(float deltaTime, const view* const parent) {
	this->updateSelf(deltaTime, parent);
	this->updateChildren(deltaTime);
}

void view::render(float deltaTime) {
	Vector2 position = this->position_;
	position += this->margin_;

	DrawRectangleRounded({position.x, position.y, this->size_.x, this->size_.y}, this->roundness(), 0, this->spec_.background);
	
	if (this->spec_.border.has_value()) {
		float borderWidth = this->spec_.border.value().width;
		DrawRectangleRoundedLinesEx({position.x + borderWidth, position.y + borderWidth, this->size_.x - borderWidth * 2, this->size_.y - borderWidth * 2}, this->roundness(), 0, this->spec_.border.value().width, this->spec_.border.value().color);
	}

	for (auto& child : this->children)
		child->render(deltaTime);
}

bool view::hovered() const {
	Vector2 mouse = GetMousePosition();
	Rectangle bounds = {this->position_.x, this->position_.y, this->size_.x, this->size_.y};

	return CheckCollisionPointRec(mouse, bounds);
}

bool view::active() const {
	return this->hovered() && IsMouseButtonDown(MOUSE_LEFT_BUTTON);
}

}