#pragma once

#include <initializer_list>
#include <variant>
#include <vector>
#include <optional>
#include <raylib.h>
#include <raymath.h>

#include "theme.hpp"

namespace mcpaint::gui {

enum class Display {
	INLINE,
	BLOCK,
	FLEX
};

struct Border {
	Color color = Theme::ELEMENT_BORDER;
	float width = 1;
};

using Length = std::variant<float, int>;
using Tuple  = std::pair<Length, Length>;

struct Spec {
	Vector2     position         = {0, 0};
	Tuple       size             = {100.0f, 100.0f};
	Color       color            = WHITE;
	Color       background       = MAGENTA;
	Color       hoverBackground  = Theme::TRANSPARENT;
	Color       hoverColor       = color;
	Color       activeBackground = Theme::ELEMENT_ACTIVE;
	float       padding          = 0;
	float       margin           = 0;
	float       gap              = 5;
	std::optional<Border> border;
	size_t      fontSize         = 12;
	size_t      radius           = 0;
	std::string font             = "assets/fonts/sf-pro.otf";
	uint8_t     flex             = 1;
	Display     display          = Display::FLEX;
};

class view {
	protected:
		std::vector<view*> children;
		Spec spec_;
		Vector2 position_;
		float margin_;

		void updateSelf(float deltaTime, const view* const parent = nullptr);
		void updateChildren(float deltaTime);
	public:
		enum class Type {
			VIEW,
			BUTTON,
			HSTACK,
			VSTACK,
			SPACER,
			ARTBOARDS
		};
		Vector2 size_;
		Vector2 renderSize;

		view(std::initializer_list<view*> children = {});
		virtual ~view();

		virtual void update(float deltaTime, const view* const parent = nullptr);
		virtual void render(float deltaTime);

		virtual bool hovered() const;
		virtual bool active() const;

		virtual view* border(Border border) {
			this->spec_.border = std::move(border);

			return this;
		}

		virtual Type type() const {
			return Type::VIEW;
		}

		virtual size_t contentWidth() const {
			return 0;
		}

		inline float roundness() const {
			return this->size_.x > this->size_.y ? this->spec_.radius / this->size_.y : this->spec_.radius / this->size_.x;
		}

		inline const Spec& spec() const {
			return this->spec_;
		}

		inline Vector2 size() const {
			return this->size_;
		}
		inline view* size(Tuple size) {
			this->spec_.size = size;
			return this;
		}

		inline void position(Vector2 position) {
			this->position_ = position;
		}
		inline Vector2 position() const {
			return this->position_;
		}

		inline view* margin(float margin) {
			this->spec_.margin = margin;
			return this;
		}

		inline view* padding(float padding) {
			this->spec_.padding = padding;
			return this;
		}

		inline view* background(Color color) {
			this->spec_.background = color;
			return this;
		}

		view(const view&) = delete;
		view& operator=(const view&) = delete;

		view(view&&) = delete;
		view& operator=(view&&) = delete;
};

inline view* View(std::initializer_list<view*> children = {}) {
	return new view(std::move(children));
}

}